@@ -1,33 +1,33 @@
-##' For each row of the input data matrix, \code{nb.glm.test} fits a
-##' NB log-linear regression model and perform large-sample tests for
+##' For each row of the input data matrix, \code{nb.glm.test} fits an
+##' NB log-linear regression model and performs large-sample tests for
 ##' a one-dimensional regression coefficient.
 ##'
 ##' \code{nbp.glm.test} provides a simple, one-stop interface to
 ##' performing a series of core tasks in regression analysis of
 ##' RNA-Seq data: it calls \code{\link{estimate.norm.factors}} to
-##' estimate normalizaton factors; it calls
-##' \code{\link{prepare.nb.data}} to create a NB data structure; it
-##' calls \code{\link{estimate.dispersion}} to estimete the NB
+##' estimate normalization factors; it calls
+##' \code{\link{prepare.nb.data}} to create an NB data structure; it
+##' calls \code{\link{estimate.dispersion}} to estimate the NB
 ##' dispersion; and it calls \code{\link{test.coefficient}} to test
-##' the regresssion coefficient.
+##' the regression coefficient.
 ##'
 ##' To keep the interface simple, \code{nbp.glm.test} provides limited
 ##' options for fine tuning models/parameters in each individual
 ##' step. For more control over individual steps, advanced users can
 ##' call \code{\link{estimate.norm.factors}},
 ##' \code{\link{prepare.nb.data}}, \code{\link{estimate.dispersion}},
-##' and \code{\link{test.coefficient}} directly, or even substitue one
+##' and \code{\link{test.coefficient}} directly, or even substitute one
 ##' or more of them with their own versions.
 ##'
 ##' @title Fit Negative Binomial Regression Model and Test for a Regression Coefficient
+##' @export
 ##' @param counts an m by n matrix of RNA-Seq read counts with rows
 ##' corresponding to gene features and columns corresponding to
 ##' independent biological samples.
-##' @param x an n by p design matrix specifiying the treatment structure.
+##' @param x an n by p design matrix specifying the treatment structure.
 ##' @param beta0 a p-vector specifying the null hypothesis. Non-NA
 ##' components specify the parameters to test and their null
-##' values. (Currently, only one-dimensional test is implemented, so
-##' only one non-NA component is allowed).
+##' values.
 ##' @param lib.sizes a p-vector of observed library sizes, usually (and by default)
 ##' estimated by column totals.
 ##' @param normalization.method a character string specifying the
@@ -37,76 +37,43 @@
 ##' normalization is applied); if \code{method="AH2010"}, the
 ##' normalization method proposed by Anders and Huber (2010) will be
 ##' used.
-##' @param dispersion.method  a character string specifying  the
-##' method for estimating the dispersion parameter. Currenlty, the
-##' only implemented option is "log-linear-rel-mean", which assumes
-##' that log dispersion is a log-linear function of the relative
-##' mean. 
+##' @param dispersion.model a character string specifying the
+##' dispersion model, and can be one of "NB2", "NBP", "NBQ" (default), "NBS" or "step".
 ##' @param tests a character string vector specifying the tests to be
 ##' performed, can be any subset of \code{"HOA"} (higher-order
-##' asymptotic test), \code{"LR"} (likelihoo ratio test), and
+##' asymptotic test), \code{"LR"} (likelihood ratio test), and
 ##' \code{"Wald"} (Wald test).
 ##' @param alternative a character string specifying the alternative
 ##' hypothesis, must be one of \code{"two.sided"} (default), \code{"greater"} or
 ##' \code{"less"}. 
+##' @param subset specify a subset of rows to perform the test on
 ##' @return  A list containing the following components: \item{data}{a
 ##' list containing the input data matrix with additional summary
 ##' quantities, output from \code{\link{prepare.nb.data}}.}
 ##' \item{dispersion}{dispersion estimates and  models, output from
 ##' \code{\link{estimate.dispersion}}.}  \item{test}{test results,
 ##' output from \code{\link{test.coefficient}}.}
-##' @author Yanming Di
-##' @examples
-##'
-##'  ## Load Arabidopsis data
-##'  data(arab);
-##'
-##'  ## Use a subset of arab data for demonstration purpose
-##'  counts = arab[1:100,];
-##'  
-##'  ## Specify treatment structure
-##'  grp.ids = as.factor(c(1, 1, 1, 2, 2, 2));
-##'  x = model.matrix(~grp.ids);
-##'
-##'  ## Specify the null hypothesis
-##'  ## The null hypothesis is beta[1]=0 (beta[1] is the log fold change).
-##'  beta0 = c(NA, 0);
-##'
-##'  ## Fit NB regression model and perform large sample tests.
-##'  ## The step can take long if the number of genes is large
-##'  fit = nb.glm.test(counts, x, beta0, normalization.method="AH2010");
-##'
-##'  ## The result contains the data, the dispersion estimates and the test results
-##'  names(fit);
-##'
-##'  ## Show top ten most differentially expressed genes
-##'  subset = order(fit$test.results$HOA$p.values)[1:10];
-##'  cbind(counts[subset,],
-##'        HOA=fit$test.results$HOA$p.values[subset],
-##'        LR=fit$test.results$LR$p.values[subset],
-##'        Wald=fit$test.results$Wald$p.values[subset]
-##'        );
-##'
+##' @example inst/examples/example.nb.glm.test.R
 nb.glm.test = function(counts, x, beta0,
-            lib.sizes = colSums(counts),
-            normalization.method = NULL,
-            dispersion.method = "log-linear-rel-mean",
-            tests=c("HOA", "LR", "Wald"),
-            alternative="two.sided") {
+  lib.sizes = colSums(counts),
+  normalization.method = "AH2010",
+  dispersion.model = "NBQ",
+  tests=c("HOA", "LR", "Wald"),
+  alternative="two.sided",
+  subset = 1:dim(counts)[1]) {
 
   ## Estimate normalization factors
   norm.factors = estimate.norm.factors(counts, lib.sizes, method=normalization.method);
   
   ## Create an NB object
-  nb.data = prepare.nb.data(counts, lib.sizes, norm.factors);
+  nb.data = prepare.nb.data(counts[subset,,drop=FALSE], lib.sizes, norm.factors);
 
   ## Estimate the dispersion parameters
-  dispersion = estimate.dispersion(nb.data, x, method=dispersion.method);
+  dispersion = estimate.dispersion(nb.data, x, model=dispersion.model);
 
   ## Test for the regression coefficient
   ## debug(test.coefficient);
   test.results = test.coefficient(nb.data, dispersion, x, beta0,
-  ##    subset = c(3890, 3891, 3892),
     tests=tests, alternative=alternative);
 
   ## Return a list summarizing the NB data (nb.data), the dispersion
@@ -116,83 +83,11 @@ nb.glm.test = function(counts, x, beta0,
   list(data = nb.data, dispersion = dispersion, test.results = test.results);
 }
 
-##' \code{estimate.norm.factors} estiamtes normalization factors to
-##' account for apparent reduction or increase in relative frequencies
-##' of non-differentially expressing genes as a result of compensating
-##' the increased or decreased relative frequencies of truly
-##' differentially expressing genes.
-##'
-##' We take gene expression to be indicated by relative frequency of
-##' RNA-Seq reads mapped to a gene, relative to library sizes (column
-##' sums of the count matrix). Since the relative frequencies sum to 1
-##' in each library (one column of the count matrix), the increased
-##' relative frequencies of truly over expressed genes in each column
-##' must be accompanied by decreased relative frequencies of other
-##' genes, even when those others do not truly differently
-##' express.
-##'
-##' The concern introduced in Robinson and Oshlack (2010) is that this
-##' reduction will give a false impression of biological relevance.
-##' Since the accommodation for relative frequencies summing to one is
-##' shared equally by a very large number of non-differentially
-##' expressing genes, we suspect that the effect is usually small, but
-##' examples where it is non-ignorable have been demonstrated
-##' (Robinson and Oshlack, 2010).
-##'
-##' A simple fix is to compute the relative frequencies relative to
-##' effective library sizes---library sizes multiplied by
-##' normalization factors.
-##'
-##' @title Estiamte Normalization Factors
-##' @param counts a matrix of RNA-Seq read counts with rows
-##' corresponding to gene features and columns corresponding to
-##' independent biological samples.
-##' @param lib.sizes  a vector of observed library sizes, usually
-##' estimated by column totals.
-##' @param method a character string specifying the method for
-##' normalization, can be NULL or "AH2010". If method=NULL, the
-##' normalization factors will have values of 1 (i.e., no
-##' normalization is applied); if method="AH2010", the normalization
-##' method proposed by Anders and Huber (2010) will be used.
-##' @references {Anders, S. and W. Huber (2010): "Differential
-##' expression analysis for sequence count data," Genome Biol., 11,
-##' R106.
-##'
-##' Robinson, M. D. and A. Oshlack (2010): "A scaling normalization method for differential expression analysis of RNA-seq data," Genome Biol., 11, R25.}
-##' @return a vector of normalization factors.
-##' @author Yanming Di
-estimate.norm.factors = function(counts, lib.sizes, method) {
-  if (is.null(method)) {
-    ## No normalization is used
-    norm.factors = rep(1, dim(counts)[2]);
-  } else {
-
-    if (method=="AH2010") {
-      ## The method proposed by Anders and Huber Genome Biology 2010,
-      ## 11:R106.
-
-      ## Create a reference column (geometric mean of the read
-      ## frequenices in each column)
-      m = exp(rowMeans(log(counts)));
-
-      ## Estimate median fold change relative to the reference column
-      size.factors = apply(counts, 2, function(y) median((y/m)[m>0]));
-
-      ## The normalization factors are defined relative to the specified library sizes
-      norm.factors =  size.factors * mean(lib.sizes) / lib.sizes;
-
-      ## Normalize the normalizaton factors
-      norm.factors = norm.factors/exp(mean(log(norm.factors)));
-    }
-  }
-
-  norm.factors
-}
-
-##' Create a NB data structure to hold the RNA-Seq read counts and
+##' Create a data structure to hold the RNA-Seq read counts and
 ##' other relevant information.
 ##'
 ##' @title Prepare the NB Data Structure for RNA-Seq Read Counts
+##' @export
 ##' @param counts an mxn matrix of RNA-Seq read counts with rows
 ##' corresponding to gene features and columns corresponding to
 ##' independent biological samples.
@@ -200,8 +95,7 @@ estimate.norm.factors = function(counts, lib.sizes, method) {
 ##' are estimated to the column totals of the matrix \code{counts}.
 ##' @param norm.factors an n-vector of normalization factors. By default, have values 1 (no normalization is applied).
 ##' @param tags a matrix of tags associated with genes, one row for
-##' each gene (having the same number of rows as \code{counts}. By
-##' default, row names of the matrix \code{counts} are used as tags.
+##' each gene (having the same number of rows as \code{counts}.
 ##' @return  A list containing the following components:
 ##' \item{counts}{the count matrix, same as input.}
 ##' \item{lib.sizes}{observed library sizes, same as input.}
@@ -209,12 +103,11 @@ estimate.norm.factors = function(counts, lib.sizes, method) {
 ##' \item{eff.lib.sizes}{effective library sizes (\code{lib.sizes} x \code{norm.factors}).}
 ##' \item{rel.frequencies}{relative frequencies (counts divided by the effective library sizes).}
 ##' \item{tags}{a matrix of gene tags, same as input.}
-##' 
-##' @author Yanming Di
 prepare.nb.data = function(counts,
   lib.sizes=colSums(counts),
   norm.factors=rep(1, dim(counts)[2]),
-  tags=matrix(row.names(counts), dim(counts)[1], 1)
+  tags=NULL
+  ##tags =  matrix(row.names(counts), dim(counts)[1], 1)
   ) {
 
   eff.lib.sizes = lib.sizes * norm.factors;
@@ -229,10 +122,36 @@ prepare.nb.data = function(counts,
     eff.lib.sizes = eff.lib.sizes,
     rel.frequencies=rel.freq,
     tags = tags);
+
+  class(nb.data) = "nb.data";
+
+  nb.data
 }
 
-##' Estimate NB dispersion by modeling it as a function of the mean
-##' frequency and library sizes.
+##' @title Print summary of the nb counts
+##' @param x output from \code{\link{prepare.nb.data}}
+##' @param ... additional parameters, currently not used
+##' @return NULL
+print.nb.data = function(x, ...) {
+
+  print(str(x));
+  print("Counts:");
+  print(head(x$counts));
+  cat("...\n");
+  print("Lirary sizes (unnormalized):");
+  print(x$lib.sizes);
+  print("Normalization factors:");
+  print(x$norm.factors);
+  print("Effective (normalized) library sizes:");
+  print(x$eff.lib.sizes);
+  print("Reads per Million:");
+  print(head(x$rel.freq * 1e6));
+  cat("...\n");
+
+  invisible();
+}
+
+##' Estimate NB dispersion by modeling it as a parametric function of preliminarily estimated log mean relative frequencies.
 ##'
 ##' We use a negative binomial (NB) distribution to model the read
 ##' frequency of gene \eqn{i} in sample \eqn{j}.  A negative binomial
@@ -244,41 +163,136 @@ prepare.nb.data = function(counts,
 ##' experiments, estimating the NB dispersion \eqn{\phi_{ij}} for each
 ##' gene \eqn{i} separately is not reliable.  One can pool information
 ##' across genes and biological samples by modeling \eqn{\phi_{ij}} as
-##' a function of the mean frequencies and library sizes. The
-##' "log-linear-rel-mean" method assumes a parametric dispersion model
-##' \deqn{\phi_{ij} = \alpha_0 + \alpha_1 \log(\pi_{ij}),} where
-##' \eqn{\pi_{ij} = \mu_{ij}/(N_j R_j)} is the relative mean frequency
-##' after normalization. The parameters \eqn{(\alpha_0, \alpha_1)} in
-##' this dispersion model are estimated by maximizing the adjusted
-##' profile likelihood.
+##' a function of the mean frequencies and library sizes.
+##'
+##' Under the NB2 model, the dispersion is a constant across all genes and samples.
+##'
+##' Under the NBP model, the log dispersion is modeled as a linear
+##' function of the preliminary estimates of the log mean relative
+##' frequencies (\code{pi.pre}):
+##'
+##' log(phi) =  par[1] + par[2] * log(pi.pre/pi.offset),
+##'
+##' where \code{pi.offset} is 1e-4.
+##'
+##' Under the NBQ model, the dispersion is modeled as a quadratic
+##' function of the preliminary estimates of the log mean relative
+##' frequencies (pi.pre):
+##'
+##' log(phi) =  par[1] + par[2] * z + par[3] * z^2,
+##'
+##' where z = log(pi.pre/pi.offset). By default, pi.offset is the median of pi.pre[subset,].
+##'
+##' Under this NBS model, the dispersion is
+##' modeled as a smooth function (a natural cubic spline function) of
+##' the preliminary estimates of the log mean relative frequencies
+##' (pi.pre).
+##'
+##' Under the "step" model, the dispersion is modeled as a step
+##' (piecewise constant) function.
 ##'
 ##' @title Estimate Negative Binomial Dispersion
-##' @param nb.data output from \code{prepare.nb.data}.
-##' @param x a design matrix specifiying the mean structure of each row.
-##' @param method the method for estimating the dispersion
-##' parameter. Currenlty, the only implemented option is
-##' "log-linear-rel-mean", which assumes that log dispersion is a
-##' log-linear function of the relative mean. 
-##' @param ... additional parameters.
-##' @return a list of two components:
-##' \item{estiamtes}{dispersion
-##' estimates for each read count, a matrix of the same dimensions as
-##' the \code{counts} matrix in nb.data.}
-##' \item{models}{a list of
-##' dispersion models, NOT intended for use by end users.}
-##' @author Yanming Di
-estimate.dispersion = function(nb.data, x, method="log-linear-rel-mean", ...) {
- 
- res = estimate.disp.mapl.nbp(nb.data$counts, nb.data$eff.lib.sizes, x,
-                         ...);
+##' @export
+##' @param nb.data output from \code{\link{prepare.nb.data}}.
+##' @param x a design matrix specifying the mean structure of each row.
+##' @param model the name of the dispersion model, one of "NB2", "NBP", "NBQ" (default), "NBS" or "step".
+##' @param method a character string specifying the method for estimating the dispersion model, one of "ML" or "MAPL" (default).
+##' @param ... (for future use).
+##' @return a list with following components:
+##' \item{estimates}{dispersion estimates for each read count, a matrix of the same dimensions as
+##' the \code{counts} matrix in \code{nb.data}.}
+##' \item{likelihood}{the likelihood of the fitted model.}
+##' \item{model}{details of the estimate dispersion model, NOT intended for use by end users. The name and contents of this component are subject to change in future versions.}
+##' @examples
+##' ## See the example for test.coefficient.
+##' @note Currently, it is unclear whether a dispersion-modeling
+##' approach will outperform a more basic approach where regression
+##' model is fitted to each gene separately without considering the
+##' dispersion-mean dependence. Clarifying the power-robustness of the
+##' dispersion-modeling approach is an ongoing research topic.
+estimate.dispersion = function(nb.data, x, model="NBQ", method="MAPL", ...) {
 
-  dispersion = list(estimates = res$phi, models = list(res));
+  ## Specify the function to be used for estimating the dispersion
+  ## model parameters
+  if (method == "MAPL") {
+    ## Maximum adjusted profile likelihood (MAPL)
+    optim.fun = optim.disp.apl
+  } else if (method == "ML") {
+    ## Maximum likelihood (ML)
+    optim.fun = optim.disp.pl
+  } else {
+    stop('method should be "MAPL" or "ML".');
+  }
+ 
+  if (model=="NBP") {
+    disp = disp.nbp(nb.data$counts, nb.data$eff.lib.sizes, x);
+
+    ## obj = list(estimates = phi, model=model, method=method, fun=disp$fun, par=res$par, z=disp$z, likelihood=-res$value);
+  } else if (model=="NBQ") {
+    disp = disp.nbq(nb.data$counts, nb.data$eff.lib.sizes, x);
+
+    ## obj = list(estimates = phi, model=model, method=method, fun=disp$fun, par=res$par, z=disp$z, likelihood=-res$value);
+  } else if (model=="NBS") {
+    disp = disp.nbs(nb.data$counts, nb.data$eff.lib.sizes, x, df=6);
+    ## obj = list(estimates = phi, model=model, method=method, fun=disp$fun, par=res$par, likelihood=-res$value);
+  } else if (model=="NB2") {
+    disp = disp.step(nb.data$counts, nb.data$eff.lib.sizes, x, df=1);
+  } else if (model=="step") {
+    disp = disp.step(nb.data$counts, nb.data$eff.lib.sizes, x);
+  } else {
+    stop('model should be "NBP", "NBQ", "NBS", "NB2" or "step".');
+  }
+
+  res = optim.fun(disp, nb.data$counts, nb.data$eff.lib.sizes, x, ...);
+
+  ##  model = c(disp, res);
+  ## class(model) = "dispersion.model";
+
+  obj = list(estimates = disp$fun(res$par), likelihood=-res$value, model= c(disp, res));
+  class(obj) = "nb.dispersion";
+
+  ## res = estimate.disp.mapl.nbp(nb.data$counts, nb.data$eff.lib.sizes, x, ...);
+  obj
+}
+
+##' @title Plot the estimated dispersion as a function of the
+##' preliminarily estimated mean relative frequencies
+##' @export
+##' @param x output from \code{\link{estimate.dispersion}}
+##' @param ... additional parameters, currently unused
+##' @return NULL
+plot.nb.dispersion = function(x, ...) {
+  plot(x$model$pi.pre, x$estimates, log="xy", xlab="Preliminary Estimates of Mean Relative Frequencies",
+       ylab ="Estiamted Dispersion");
+  invisible();
+}
+
+##' @title Print the estimated dispersion model
+##' @export
+##' @param x output from from \code{\link{estimate.dispersion}}
+##' @param ... additional parameters, currently unused
+##' @return  NULL
+print.nb.dispersion = function(x, ...) {
+
+  print(str(x));
+  
+  model = x$model;
+
+  model$pi.pre = NULL;
+  model$subset = NULL;
+  print.default(model);
+
+  print("Likelihood of the fitted dispersion model:");
+  print(head(x$likelihood));
+  print("Dispersion Estimates:");
+  print(head(x$estimates));
+
+  invisible();
 }
 
 ##' \code{test.coefficient} performs large-sample tests (higher-order
-##' asymptotic test, likelihood ratio test, and/or Wald tests) for
-##' testing one of the regression coefficient in an NB regression
-##' model. 
+##' asymptotic test, likelihood ratio test, and/or Wald test) for
+##' testing regression coefficients in an NB regression model. 
 ##'
 ##' \code{test.coefficient} performs large-sample tests for a
 ##' one-dimensional (\eqn{q=1}) component \eqn{\psi} of the
@@ -305,11 +319,12 @@ estimate.dispersion = function(nb.data, x, method="log-linear-rel-mean", ...) {
 ##' corresponding unadjusted likelihood ratio tests, especially in
 ##' situations where the sample size is small and/or when the number
 ##' of nuisance parameters (\eqn{p-q}) is large. The implementation
-##' here is based on Skovgaard (2001). See Di et al. 2012 for more
+##' here is based on Skovgaard (2001). See Di et al. 2013 for more
 ##' details.
 ##'
 ##' @title Large-sample Test for a Regression Coefficient in an
 ##' Negative Binomial Regression Model
+##' @export
 ##' @param nb an NB data object, output from \code{\link{prepare.nb.data}}.
 ##' @param dispersion dispersion estimates, output from \code{\link{estimate.disp}}.
 ##' @param x an \eqn{n} by \eqn{p} design matrix describing the treatment structure
@@ -319,12 +334,12 @@ estimate.dispersion = function(nb.data, x, method="log-linear-rel-mean", ...) {
 ##' only one non-NA component is allowed).
 ##' @param tests a character string vector specifying the tests to be
 ##' performed, can be any subset of \code{"HOA"} (higher-order
-##' asymptotic test), \code{"LR"} (likelihoo ratio test), and
+##' asymptotic test), \code{"LR"} (likelihood ratio test), and
 ##' \code{"Wald"} (Wald test).
 ##' @param alternative a character string specifying the alternative
 ##' hypothesis, must be one of \code{"two.sided"} (default),
 ##' \code{"greater"} or \code{"less"}. 
-##' @param subset an index vector specifying on which rows should be
+##' @param subset an index vector specifying on which rows should the 
 ##' tests be performed
 ##' @param print.level a number controlling the amount of messages
 ##' printed: 0 for suppressing all messages, 1 (default) for basic
@@ -340,7 +355,11 @@ estimate.dispersion = function(nb.data, x, method="log-linear-rel-mean", ...) {
 ##' Skovgaard, I. (2001): "Likelihood asymptotics," Scandinavian
 ##' Journal of Statistics, 28, 3-32.
 ##'
-##' @return a list containig the following components:
+##' Di Y, Schafer DW, Emerson SC, Chang JH (2013): "Higher order
+##' asymptotics for negative binomial regression inferences from
+##' RNA-sequencing data". Stat Appl Genet Mol Biol, 12(1), 49-70.
+##' 
+##' @return a list containing the following components:
 ##' \item{beta.hat}{an \eqn{m} by \eqn{p} matrix of regression
 ##' coefficient under the full model} \item{mu.hat}{an \eqn{m} by
 ##' \eqn{n} matrix of fitted mean frequencies under the full model}
@@ -351,10 +370,9 @@ estimate.dispersion = function(nb.data, x, method="log-linear-rel-mean", ...) {
 ##' \code{p.values} and \code{q.values}, giving p-values and q-values
 ##' of the corresponding tests when that test is included in
 ##' \code{tests}.}
-##'
-##' @author Yanming Di
+##' @example inst/examples/example.test.coefficient.R
 test.coefficient = function(nb, dispersion, x, beta0,
-  tests,
+  tests = c("HOA", "LR", "Wald"),
   alternative="two.sided",
   subset = 1:m,
   print.level=1) {
@@ -389,33 +407,38 @@ test.coefficient = function(nb, dispersion, x, beta0,
     mu.tilde = matrix(NA, m, n));
 
   if ("HOA" %in% tests) {
-    HOA = list(p.values=rep(NA, m), q.values=rep(NA, m));
+    HOA = data.frame(statistic = rep(NA, m), p.values=rep(NA, m), q.values=rep(NA, m));
   }
 
   if ("LR" %in% tests) {
-    LR = list(p.values=rep(NA, m), q.values=rep(NA, m));
+    LR = data.frame(statistic = rep(NA, m), p.values=rep(NA, m), q.values=rep(NA, m));
   }
 
   if ("Wald" %in% tests) {
-    Wald = list(p.values=rep(NA, m), q.values=rep(NA, m));
+    Wald = data.frame(statistic = rep(NA, m), p.values=rep(NA, m), q.values=rep(NA, m));
+  }
+
+  if ("score" %in% tests) {
+    score = data.frame(statistic = rep(NA, m), p.values=rep(NA, m), q.values=rep(NA, m));
   }
 
   if (print.level>1) pb = txtProgressBar(style=3);
+
   for (i in subset) {
     if (print.level>1) setTxtProgressBar(pb, i/m);
 
     if (nh==1) {
-    res.hoa = try(
-      hoa.1d(counts[i,], lib.sizes, x, phi[i,], beta0,
-             alternative=alternative,
-             print.level=print.level-1),
-      silent=TRUE); 
-  } else {
-    res.hoa = try(
-      hoa.hd(counts[i,], lib.sizes, x, phi[i,], beta0,
-             print.level=print.level-1),
-      silent=TRUE); 
-  }
+      res.hoa = try(
+        hoa.1d(counts[i,], lib.sizes, x, phi[i,], beta0,
+               alternative=alternative,
+               print.level=print.level-1),
+        silent=TRUE); 
+    } else {
+      res.hoa = try(
+        hoa.hd(counts[i,], lib.sizes, x, phi[i,], beta0,
+               print.level=print.level-1),
+        silent=TRUE); 
+    }
 
     if (!("try-error" %in% class(res.hoa))) {
       ## print(i);
@@ -425,16 +448,32 @@ test.coefficient = function(nb, dispersion, x, beta0,
       obj$mu.tilde[i,] = res.hoa$mu.tilde;
 
       if ("HOA" %in% tests) {
+        if (nh==1) {
+          HOA$statistic[i] = res.hoa$rstar;
+        } else {
+          HOA$statistic[i] = res.hoa$lambda.star;
+        }
         HOA$p.values[i] = res.hoa$pstar;
       }
       
       if ("LR" %in% tests) {
+        if (nh==1) {
+          LR$statistic[i] = res.hoa$r;
+        } else {
+          LR$statistic[i] = res.hoa$lambda;
+        }
         LR$p.values[i] = res.hoa$p;
       }
 
       if ("Wald" %in% tests) {
+        LR$statistic[i] = res.hoa$w;
         Wald$p.values[i] = res.hoa$p.wald;
       }
+
+      if ("score" %in% tests) {
+        score$statistic[i] = res.hoa$u;
+        score$p.values[i] = res.hoa$p.score;
+      }
     }
   }
 
@@ -462,6 +501,29 @@ test.coefficient = function(nb, dispersion, x, beta0,
     obj$Wald = Wald;
   }
 
+  if ("score" %in% tests) {
+    score$q.values = compute.q.values(score$p.values);
+    obj$score = score;
+  }
+
+  obj$x = x;
+  obj$beta0 = beta0;
+  obj$eff.lib.sizes = nb$eff.lib.sizes;
+
+  class(obj) = "nb.test";
+
   obj;
 }
 
+##' We simply print out the structure of \code{x}. (Currenlty the
+##' method is equivalent to \code{print(str(x))}.)
+##'
+##' @title Print output from \code{\link{test.coefficient}}
+##' @param x output from \code{\link{test.coefficient}}
+##' @param ... currenty not used
+##' @return NULL
+print.nb.test = function(x, ...) {
+  print(str(x));
+  invisible();
+}
+
